{ ExitWin version 2.0 copyright  1996-1997 by Alexander Meeder
  Encapsulates the Win32 API "ExitWindowsEx' function
  Very easy to use, just place it on your form, choose the appropriate action
  with the ExitWinType-property, call the ExitWin1.Execute method which returns
  false if something went wrong. You can then use the LastError propertyto
  determine the problem

  Release info:
    Version 1.0 - First release (1996)
    Version 1.1 - Release with aditional functionality build in by Paul W. Guilmette
    Version 2.0 - Second release (1997), after some usefull tips this component
                  is Windows NT-safe so you can use it to shutdown Windows NT

 please mail me if you use it or have some suggestions: ameeder@dds.nl

 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

 Paul W. Guilmette (tridata@gte.net) -- Thought this component could use a
 couple of modifications New Properties
 List Of Exits : Boolean   -- If True will display to the user the ways to exit
                              If False will use the exit win type selected
 ListTop                   -- Top Position of the display list
 ListLeft                  -- Left Position of the List

 Torben Falck (tof@post6.tele.dk) -- Found out that when using this control
                 under Windows NT you cannot shutdown the system when you don't
                 have the privilege to do so. He suggested to add some functions
                 to set this privilege
}

unit ExitWin;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, TypeExit;

Type
  // Different OS platforms
  TWinType = (wtWindows95, wtWindowsNT, wtWin32s, wtUnknown);

  // Different options to ShutDown/Logoff Windows95/NT
  TExitWinType  = (ewLogoff, ewReboot, ewShutDown, ewPowerOff);

  // New component, encapsulates Win32 API ExitWindowsEx
  TExitWin = class(TComponent)
  private
    { Private declarations }
    FExitWinType  : TExitWinType;
    FUnconditional: boolean;
    FLastError    : string;
    FShowDialog   : boolean;
    FListTop      : Integer;
    FListLeft     : Integer;
    procedure SetExitWinType(Value: TExitWinType);
    function GetWinType: TWinType;
    function CanShutDown: boolean;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Execute: boolean;
    property WindowsType: TWinType read GetWinType;
  published
    { Published declarations }
    property ExitWinType   : TExitWinType read FExitWinType   write SetExitWinType default EWReboot;
    property LastError     : string       read FLastError;
    property ListTop       : integer      read FListTop       write FlistTop       default 220;
    property ListLeft      : integer      read FListLeft      write FListLeft      default 250;
    property ShowDialog    : boolean      read FShowDialog    write FShowDialog    default false;
    property Unconditional : boolean      read FUnconditional write FUnconditional default false;
  end;
procedure Register;

implementation

// Override 'create-method'
constructor TExitWin.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FExitWinType   := ewLogoff;
  FListTop       := 220;
  FListLeft      := 250;
  FShowDialog    := false;
  FUnconditional := false;
end;

// Override 'destroy-method'
destructor TExitWin.Destroy;
begin
  inherited Destroy;
end;

// Sets privileges of program under Windows NT to SE_SHUTDOWN_NAME...
function TExitWin.CanShutDown: boolean;
const
  SE_SHUTDOWN_NAME = 'SeShutDownPrivilege';   // Borland forgot this declaration
var
  hToken: THandle;
  tkp: TTokenPrivileges;
  retval: integer;
begin
  Result := False;
  if not (OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES or
     TOKEN_QUERY, hToken)) then
     begin
       FLastError := 'Error: ' + 'OpenProcessToken';
       Exit;
     end;

  LookupPrivilegeValue(nil, SE_SHUTDOWN_NAME, tkp.Privileges[0].Luid);
  tkp.PrivilegeCount := 1;
  tkp.Privileges[0].Attributes := SE_PRIVILEGE_ENABLED;

  AdjustTokenPrivileges(hToken, false, tkp, 0, tkp, retval);
  if GetLastError() <> ERROR_SUCCESS then
  begin
    FLastError := 'Error: ' + 'AdjustTokenPrivileges';
    Exit;
  end
  else
    Result := True;
end;

// Property ExitWinType procedure
procedure TExitWin.SetExitWinType(Value: TExitWinType);
begin
  if Value <> FExitWinType then
  begin
    FExitWinType := Value;
  end;
end;

// Get the Windowsplatform information...
function TExitWin.GetWinType: TWinType;
var
  VersionInfo: TOSVersionInfo;
begin
  Result := wtUnknown;
  VersionInfo.dwOSVersionInfoSize := SizeOf(VersionInfo);
  GetVersionEx(VersionInfo);
  case VersionInfo.dwPlatformId of
    VER_PLATFORM_WIN32S        : Result := wtWin32s;
    VER_PLATFORM_WIN32_WINDOWS : Result := wtWindows95;
    VER_PLATFORM_WIN32_NT      : Result := wtWindowsNT;
  end;
end;

// The functionality of this component is build in this routine
function TExitWin.Execute: boolean;
var
  Flags: word;
begin
  Result := False;
  Flags := 0;

  if FShowDialog then
  begin
    PWGTypeExit    := TPWGTypeExit.Create(Application);
    {Set The Form Locating}
    with PWGTypeExit do
    begin
      Left   := FListLeft;
      Top    := FListTop;
    end;
    {Set The Logoff setting}
    PWGTypeExit.ShowModal;
    case HowExit of
      mrLogoff  : FExitWinType := ewLogoff;
      mrReboot  : FExitWinType := ewReboot;
      mrShutdown: FExitWinType := ewShutDown;
      mrPoweroff: FExitWinType := ewPoweroff;
      mrCancel  : Exit;
    end;
    if PWGTypeExit.Unconditional.Checked then Unconditional := True;
    PWGTypeExit.Release;
  end;

  case FExitWinType of
    ewLogoff   : Flags := EWX_LOGOFF;
    ewReboot   : Flags := EWX_REBOOT;
    ewShutDown : Flags := EWX_SHUTDOWN;
    ewPowerOff : Flags := EWX_POWEROFF;
  end;

  if Unconditional then Flags := Flags or EWX_FORCE;

  if WindowsType = wtWindowsNT then
  begin
    if CanShutDown then Result := ExitWindowsEx(Flags, 0);
  end
  else
    ExitWindowsEx(Flags, 0);

  // If somthing went wrong, set property 'LastError'
  if not Result then
     FLastError := 'Error: ' + IntToStr(GetLastError);
end;

procedure Register;
begin
  RegisterComponents('Freeware', [TExitWin]);
end;

end.